// defines below to control which Tonemap to use
// If none of these are defined, no additional tonemapping is done
// Remove temp/shaders in userfolder after every change and restart the game

//#define BNG_ACES_TONEMAP
#define PBR_NEUTRAL_TONEMAP
//#define UNCHARTED_2_TONEMAP



#ifdef BNG_ACES_TONEMAP
#include "shaders/common/postfx/hdr/ACES.h.hlsl"
#endif

// Uncharted
// http://de.slideshare.net/ozlael/hable-john-uncharted2-hdr-lighting
float3 TonemapOperatorUncharted2(float3 x)
{
   float A = 0.15;
   float B = 0.50;
   float C = 0.10;
   float D = 0.20;
   float E = 0.02;
   float F = 0.30;

   return ((x*(A*x+C*B)+D*E)/(x*(A*x+B)+D*F))-E/F;
}

float3 TonemapUncharted2(float3 color)
{
    const float exposure_adjustment = 1; //16;
    const float exposure_bias = 1.25; //2;

    color = max(color, 0) * exposure_adjustment;
    const float whiteScale = (1.0f / TonemapOperatorUncharted2(11.2f)).x;
    return saturate(TonemapOperatorUncharted2(exposure_bias * color) * whiteScale);
}

// Khronos Group PBR Neutral (Apache 2.0 License)
// Source: https://github.com/KhronosGroup/ToneMapping/tree/main/PBR_Neutral
// Changes:
// - convert from glsl to hlsl
// LICENSE.md applies

// Input color is non-negative and resides in the Linear Rec. 709 color space.
// Output color is also Linear Rec. 709, but in the [0, 1] range.

float3 PBRNeutralToneMapping( float3 color ) {
  const float startCompression = 0.8 - 0.04;
  const float desaturation = 0.15;

  float x = min(color.r, min(color.g, color.b));
  float offset = x < 0.08 ? x - 6.25 * x * x : 0.04;
  color -= offset;

  float peak = max(color.r, max(color.g, color.b));
  if (peak < startCompression) return color;

  const float d = 1. - startCompression;
  float newPeak = 1. - d * d / (peak + d - startCompression);
  color *= newPeak / peak;

  float g = 1. - 1. / (desaturation * (peak - newPeak) + 1.);
  return lerp(color, newPeak * float3(1, 1, 1), g);
}

// function that's called for tonemapping
float3 tonemap(float3 rgb) {
#ifdef BNG_ACES_TONEMAP
    return ACESFitted(rgb);
#else
    #ifdef PBR_NEUTRAL_TONEMAP
        return PBRNeutralToneMapping(rgb);
    #else
        #ifdef UNCHARTED_2_TONEMAP
            return TonemapUncharted2(rgb);
        #else
            return rgb;
        #endif
    #endif
#endif
}